@@ -192,8 +192,10 @@ gail_button_class_init (GailButtonClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
   GailWidgetClass *widget_class;
+  GailContainerClass *container_class;
 
   widget_class = (GailWidgetClass*)klass;
+  container_class = (GailContainerClass*)klass;
 
   gobject_class->finalize = gail_button_finalize;
 
@@ -204,6 +206,9 @@ gail_button_class_init (GailButtonClass *klass)
   class->ref_child = gail_button_ref_child;
   class->ref_state_set = gail_button_ref_state_set;
   class->initialize = gail_button_real_initialize;
+
+  container_class->add_gtk = NULL;
+  container_class->remove_gtk = NULL;
 }
 
 static void
@@ -231,8 +236,6 @@ gail_button_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  set_role_for_button (accessible, widget);
-
   return accessible;
 }
 
@@ -276,19 +279,9 @@ gail_button_real_initialize (AtkObject *obj,
   GailButton *button = GAIL_BUTTON (obj);
   guint handler_id;
   GtkWidget  *label;
-  const gchar *label_text;
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  /*
-   * As we report the button as having no children we are not interested
-   * in add and remove signals
-   */
-  handler_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (obj), "gail-add-handler-id"));
-  g_signal_handler_disconnect (data, handler_id);
-  handler_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(obj), "gail-remove-handler-id"));
-  g_signal_handler_disconnect (data, handler_id);
-
   button->state = GTK_STATE_NORMAL;
 
   g_signal_connect (data,
@@ -314,6 +307,9 @@ gail_button_real_initialize (AtkObject *obj,
     {
       gail_button_init_textutil (button, label);
     }
+
+  set_role_for_button (obj, data);
+
 }
 
 static void
@@ -409,7 +405,7 @@ gail_button_do_action (AtkAction *action,
 	}
       g_queue_push_head (button->action_queue, (gpointer) i);
       if (!button->action_idle_handler)
-	button->action_idle_handler = gtk_idle_add (idle_do_action, button);
+	button->action_idle_handler = g_idle_add (idle_do_action, button);
       break;
     default:
       return_value = FALSE;
@@ -767,16 +763,21 @@ get_image_from_button (GtkWidget *button)
   GtkImage *image = NULL;
 
   child = gtk_bin_get_child (GTK_BIN (button));
-  if (GTK_IS_ALIGNMENT (child))
-    child = gtk_bin_get_child (GTK_BIN (child));
-  if (GTK_IS_CONTAINER (child))
+  if (GTK_IS_IMAGE (child))
+    image = GTK_IMAGE (child);
+  else
     {
-      list = gtk_container_get_children (GTK_CONTAINER (child));
-      if (!list)
-        return NULL;
-      if (GTK_IS_IMAGE (list->data))
-        image = GTK_IMAGE (list->data);
-      g_list_free (list);
+      if (GTK_IS_ALIGNMENT (child))
+        child = gtk_bin_get_child (GTK_BIN (child));
+      if (GTK_IS_CONTAINER (child))
+        {
+          list = gtk_container_get_children (GTK_CONTAINER (child));
+          if (!list)
+            return NULL;
+          if (GTK_IS_IMAGE (list->data))
+            image = GTK_IMAGE (list->data);
+          g_list_free (list);
+        }
     }
 
   return image;
@@ -1255,7 +1256,7 @@ gail_button_finalize (GObject            *object)
   g_free (button->click_keybinding);
   if (button->action_idle_handler)
     {
-      gtk_idle_remove (button->action_idle_handler);
+      g_source_remove (button->action_idle_handler);
       button->action_idle_handler = 0;
     }
   if (button->action_queue)
@@ -1280,6 +1281,9 @@ get_label_from_button (GtkWidget *button,
   if (index > 0 && !allow_many)
     g_warning ("Inconsistent values passed to get_label_from_button");
 
+  if (GTK_IS_OPTION_MENU (button))
+    return NULL;
+
   n_labels = 0;
   child = gtk_bin_get_child (GTK_BIN (button));
   if (GTK_IS_ALIGNMENT (child))
