@@ -53,6 +53,13 @@ GType gail_container_get_type (void);
 struct _GailContainerClass
 {
   GailWidgetClass parent_class;
+
+  gint (*add_gtk) (GtkContainer *container,
+		   GtkWidget	*widget,
+		   gpointer	data);
+  gint (*remove_gtk) (GtkContainer *container,
+                      GtkWidget	   *widget,
+                      gpointer	   data);
 };
 
 AtkObject* gail_container_new (GtkWidget *widget);
