@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -52,6 +52,7 @@ static void      _gail_toplevel_remove_child            (GailToplevel
                                                         GtkWindow               *window);
 static gboolean  is_tooltip_window                      (GtkWidget              *widget);
 static gboolean  is_attached_menu_window                (GtkWidget              *widget);
+static gboolean  is_combo_window                        (GtkWidget              *widget);
 
 
 static gpointer parent_class = NULL;
@@ -226,6 +227,7 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
   GObject *object;
   GtkWidget *widget;
   gint n_children;
+  AtkObject *child;
 
   object = g_value_get_object (param_values + 0);
 
@@ -235,6 +237,7 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
   widget = GTK_WIDGET (object);
   if (widget->parent || 
       is_attached_menu_window (widget) ||
+      is_combo_window (widget) ||
       GTK_IS_PLUG (widget) || 
       is_tooltip_window (widget))
     return TRUE;
@@ -248,9 +251,11 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
   * Must subtract 1 from the n_children since the index is 0-based
   * but g_list_length is 1-based.
   */
+  child = gtk_widget_get_accessible (widget);
+  atk_object_set_parent (child, atk_obj);
   g_signal_emit_by_name (atk_obj, "children-changed::add",
                          n_children - 1, 
-                         gtk_widget_get_accessible (widget), NULL);
+                         child, NULL);
 
   /* Connect destroy signal callback */
   g_signal_connect (G_OBJECT(object), 
@@ -276,7 +281,7 @@ gail_toplevel_hide_event_watcher (GSignalInvocationHint *ihint,
   object = g_value_get_object (param_values + 0);
 
   if (!GTK_IS_WINDOW (object))
-    return FALSE;
+    return TRUE;
 
   _gail_toplevel_remove_child (toplevel, GTK_WINDOW (object));
   return TRUE;
@@ -292,6 +297,7 @@ _gail_toplevel_remove_child (GailToplevel *toplevel,
   AtkObject *atk_obj = ATK_OBJECT (toplevel);
   GList *l;
   guint window_count = 0;
+  AtkObject *child;
 
   if (toplevel->window_list)
     {
@@ -307,9 +313,11 @@ _gail_toplevel_remove_child (GailToplevel *toplevel,
               /* Remove the window from the window_list & emit the signal */
               toplevel->window_list = g_list_remove (toplevel->window_list,
                                                      l->data);
+              child = gtk_widget_get_accessible (GTK_WIDGET (window));
               g_signal_emit_by_name (atk_obj, "children-changed::remove",
                                      window_count, 
-                                     gtk_widget_get_accessible (GTK_WIDGET (window)), NULL);
+                                     child, NULL);
+              atk_object_set_parent (child, NULL);
               break;
             }
 
@@ -341,9 +349,38 @@ is_attached_menu_window (GtkWidget *widget)
       GtkWidget *attach;
 
       attach = gtk_menu_get_attach_widget (GTK_MENU (child));
-      if (GTK_IS_MENU_ITEM (attach))
+      if (GTK_IS_MENU_ITEM (attach) ||
+          GTK_IS_OPTION_MENU (attach))
         ret = TRUE;
     }
   return ret;
 }
 
+static gboolean
+is_combo_window (GtkWidget *widget)
+{
+  GtkWidget *child = GTK_BIN (widget)->child;
+  AtkObject *obj;
+  GtkAccessible *accessible;
+
+  if (!GTK_IS_EVENT_BOX (child))
+    return FALSE;
+
+  child = GTK_BIN (child)->child;
+
+  if (!GTK_IS_FRAME (child))
+    return FALSE;
+
+  child = GTK_BIN (child)->child;
+
+  if (!GTK_IS_SCROLLED_WINDOW (child))
+    return FALSE;
+
+  obj = gtk_widget_get_accessible (child);
+  obj = atk_object_get_parent (obj);
+  accessible = GTK_ACCESSIBLE (obj);
+  if (GTK_IS_COMBO (accessible->widget))
+    return TRUE;
+
+  return  FALSE;
+}
