@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -278,8 +278,8 @@ gail_key_snooper (GtkWidget *the_widget, GdkEventKey *event, gpointer func_data)
       info->func_data = func_data;
       consumed = g_hash_table_foreach_steal (new_hash, notify_hf, info);
       g_hash_table_destroy (new_hash);
+      g_free (info->key_event);
     }
-  g_free (info->key_event);
   g_free (info);
   return (consumed ? 1 : 0);
 }
@@ -413,6 +413,7 @@ state_event_watcher (GSignalInvocationHint  *hint,
   GObject *object;
   GtkWidget *widget;
   AtkObject *atk_obj;
+  AtkObject *parent;
   GdkEventWindowState *event;
   gchar *signal_name;
   guint signal_id;
@@ -445,8 +446,12 @@ state_event_watcher (GSignalInvocationHint  *hint,
   
   atk_obj = gtk_widget_get_accessible (widget);
   g_return_val_if_fail (GAIL_IS_WINDOW (atk_obj), FALSE);
-  signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
-  g_signal_emit (atk_obj, signal_id, 0);
+  parent = atk_object_get_parent (atk_obj);
+  if (parent == atk_get_root ())
+    {
+      signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
+      g_signal_emit (atk_obj, signal_id, 0);
+    }
 
   return TRUE;
 }
@@ -506,6 +511,7 @@ configure_event_watcher (GSignalInvocationHint  *hint,
   GObject *object;
   GtkWidget *widget;
   AtkObject *atk_obj;
+  AtkObject *parent;
   GdkEvent *event;
   gchar *signal_name;
   guint signal_id;
@@ -536,8 +542,12 @@ configure_event_watcher (GSignalInvocationHint  *hint,
 
   atk_obj = gtk_widget_get_accessible (widget);
   g_return_val_if_fail (GAIL_IS_WINDOW (atk_obj), FALSE);
-  signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
-  g_signal_emit (atk_obj, signal_id, 0);
+  parent = atk_object_get_parent (atk_obj);
+  if (parent == atk_get_root ())
+    {
+      signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
+      g_signal_emit (atk_obj, signal_id, 0);
+    }
 
   return TRUE;
 }
