@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -208,6 +208,8 @@ gail_widget_real_initialize (AtkObject *obj,
                     NULL);
   g_object_set_data (G_OBJECT (obj), "atk-component-layer",
 		     GINT_TO_POINTER (ATK_LAYER_WIDGET));
+
+  obj->role = ATK_ROLE_UNKNOWN;
 }
 
 AtkObject* 
@@ -223,8 +225,6 @@ gail_widget_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_UNKNOWN;
-
   return accessible;
 }
 
@@ -920,7 +920,17 @@ static gboolean gail_widget_on_screen (GtkWidget *widget)
         return_value = TRUE;
     }
   else
-    return_value = TRUE;
+    {
+      /*
+       * Check whether the widget has been placed of the screen. The
+       * widget may be MAPPED as when toolbar items do not fit on the toolbar.
+       */
+      if (widget->allocation.x + widget->allocation.width <= 0 &&
+          widget->allocation.y + widget->allocation.height <= 0)
+        return_value = FALSE;
+      else 
+        return_value = TRUE;
+    }
 
   return return_value;
 }
